
#include "map.h"            // processor register defines
#include "sci.h"            // Bootloader's SCI included
#include "config.h"         // Flash API declarations


#define BYTE unsigned char
#define WORD unsigned int

#define TOUPPER(x) ((BYTE)(x & 0xDF))
#define EnableInterrupts()  asm CLI
#define DisableInterrupts() asm SEI

// clock speed
#define BUS_CLOCK_HZ 3200000

/////////////////////////////////////////////////////
// I/O mappings

#define IO_LED    PTA5
#define SDA 	PTA3
#define SDA_DDR 	DDRA_BIT3
#define SCL 	PTA4
#define OUTPUT 	PTA1				

/////////////////////////////////////////////////////

#define RESET_ADR	0		// address of RESET value (YES/NO) in the memory
#define DATA_START	1		// address where data starts in the memory
#define PERIOD	2			// period in 0.1 sec, 10 = 1sec
#define MAX_MEM	2047
#define STOP_BYTE 127		// at the end of the data


#pragma DATA_SEG SHORT MY_ZEROPAGE

BYTE chr;
BYTE Tick;
BYTE Delta;
unsigned int x;
unsigned int i,Pressure, Previous_pressure, Mem_address;

#pragma DATA_SEG DEFAULT


void Init(void)
{
#ifdef COP_ENABLE
    CONFIG1 = 0x00;
#else
    CONFIG1 = CONFIG1_COPD;
#endif
    
    DDRA = 0xFF;			// all outputs
    DDRA_BIT0 = 0;			// PTA0 jako input
    PTAPUE = 0x7F;			// all pull ups
    
    TSC_PS2 = 1;
    TSC_PS1 = 1;
    TSC_PS0 = 0;			// prescaller /64
      
	TSC_TOIE = 1;			// enables TIM interrupt
	TSC_TSTOP = 0;			// runs timer

    EnableInterrupts();
    
    
}

interrupt IV_IRQ1 void Int_IRQ(void)
{
//    ISCR_ACK1 = 1;              /* IRQ ack */
}

interrupt IV_TCHO void Int_TimerCH0(void)
{
//    TSC0;
//    TSC0_CH0F = 0;	        /* clearing CH0 flag */

}

interrupt IV_TCH1 void Int_TimerCH1(void)
{
//    TSC1;
//    TSC1_CH1F = 0;	        /* clearing CH1 flag */
}

interrupt IV_TOVF void Int_TimerTOV(void)
{
    TSC;
    TSC_TOF = 0;              /* clearing TOV flag */

    TMODH = 0x13;				
    TMODL = 0x88;			// 100ms rate preset
	Tick++;

//#define TIMPERIOD (BUS_CLOCK_HZ/(64*10))   /* 64 prescaler, 10x za sekundu ;o) */
//    TMOD = TIMPERIOD;  // 


}

interrupt IV_KBRD void Int_Keyboard(void)
{
//    KBSCR_ACKK = 1;         /* keyb int. acknowledge */

}

interrupt IV_ADC void Int_ADConv(void)
{
BYTE temp;

//    temp = ADR;             /* just read ADR or write to ADSCR to clear int. */

}

/*-------------------------------------------------------*/

#define TERM_Write SCIAPIWrite


/*-------------------------------------------------------*/
void Delay_5us(void)
{
	asm 
	{
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	}

}

void Delay(unsigned int HowMuch)
{
unsigned int m;

	for (m=0;m<HowMuch;m++)
	{
		Delay_5us();
	}

}

/*-------------------------------------------------------*/
void I2C_start(void)	/* I2C bus start condition */
{
	SCL = 1;
	SDA = 1;
	Delay_5us();
	SDA = 0;
	Delay_5us();
	SCL = 0;
}
/*-------------------------------------------------------*/
void I2C_stop(void)	/* I2C bus stop condition */
{
	SDA = 0;
	Delay_5us();
	SCL = 1;
	Delay_5us();
	SDA = 1;
}
/*-------------------------------------------------------*/
BYTE I2C_write(BYTE Xvalue)
{
BYTE i,err=0;

	for (i=0x80;i>0;i/=2) 		//shift bit for masking
	{ if (i & Xvalue) SDA=1; 	//masking value with i , write to bus
		else SDA=0;
		asm						// wait abt 1us
		{
		 nop
		 nop
		 nop
		}

		SCL=1; 					//clk for I2C
		Delay_5us();
		SCL=0;
	}

	SDA=1; 						//release SDA line
	SDA_DDR = 0;				// configure as input
	Delay_5us();
	SCL=1; 						//clk #9 for ack
	Delay_5us();
	err=SDA; 					//check ack 
	SCL=0;
	SDA_DDR = 1;				// configure as out again
	return err; 				//err=1 in case of no acknowledge
}
/*-------------------------------------------------------*/

BYTE I2C_read(BYTE ack)
{

BYTE val=0;
unsigned int i=0;

	SDA=1; 						//release SDA-line
	SDA_DDR = 0;				// configure as input
	for (i=0x80;i>0;i/=2) 		//shift bit for masking
	{
		SCL=1; 					//clk
		Delay_5us();
		if (SDA) val=(val | i); //read bit
		SCL=0;
		Delay_5us();
	}
	Delay_5us();
	SDA_DDR = 1;				// configure as output
	
	if (ack == 1)				// make ACK from master
	{
		SDA = 0;
		Delay_5us();
		SCL = 1;
		Delay_5us();
		SCL = 0;
	}
	return val;

}

/*-------------------------------------------------------------------------------
EEPROM preparation
---------------------------------------------------------------------------------*/

void Priprava (unsigned int Adresa)
{

	I2C_start ();										// Start condition
	I2C_write( 0xA0 | ( 0x0E & ( Adresa >> 7)));		// memory's address and 3 MSBs of address, LSB = 0 => write
	I2C_write (Adresa);									// lower part of address 

}


void putuint(unsigned int uintval, char len)
{
char i, buff[] = "    ";

    for (i=0; i<len; i++)
    {
	    buff[len-1-i] = (char)('0' + (int)uintval%10);   // w/o this (int) it's buggy ...
    	uintval /= 10;
    }
  
  for (i=0; i<len; i++)				//sends to RS232
  {
    TERM_Write((buff[i]));
  }
  TERM_Write(13);
}




/*-----------------------------------------------------*/
unsigned int Measure_ADC(void)
{
BYTE temp;
unsigned int Result=0;

	I2C_start();
	I2C_write(0x91);				// ADC address, then read
	Result = I2C_read(1);					// hi byte 
	Result = ((Result << 8) + I2C_read(1));	// lo byte 
	temp = I2C_read(1);						// config register, not used
	I2C_stop();
	return (Result);

}

/*-----------------------------------------------------*/
void Measure_and_write(void)
{


	Pressure = Measure_ADC();
	Delta = Previous_pressure - Pressure;
	if (Delta >= 127) Delta = Delta - 1;		// correction -1 for negative values (255 is prohibited)
	Write_FLASH(Mem_address, Delta);
	Mem_address++;
	Previous_pressure = Pressure;
	

}
/*-----------------------------------------------------*/
void Sends_data(void)						// reads data from memory 
{											// and send it to serial port

BYTE temp = 0;


	putuint(126,3);						// start byte
	Delay(10000);
	putuint(3,3);						// altimeter type
	Delay(10000);

	putuint(PERIOD,3);					// period value

	for (i = 0; i < 10; i++)
	{
		putuint(1,3);					// kompatibilita
	}

	while (temp != 255)					// wait for empty cell
	{
		temp = Read_FLASH(x);
		putuint(temp,3);
		x++;
	}
	putuint(127,3);
	putuint(128,3);

}

/*-----------------------------------------------------*/

void Blik_LED(void)
{

	for(;;)
	{	
		IO_LED = 0;
		for (i = 1; i < 2000; i++)
		{
			Delay_5us();
		}
		IO_LED = 1;
		for (i = 1; i < 2000; i++)
		{
			Delay_5us();
		}

	}
}
/*-----------------------------------------------------*/
void main(void) 
{
    Init();
 Delay_5us();
 
   SCIAPIInit();

#ifdef COP_ENABLE
        COPCTL = 0;             /* bump watchdog */
#endif

    DDRA_BIT1 = 0;							// PTA1 as input for moment

	if (OUTPUT == 0)						// is reset done by external jumper?
	{
		Erase_FLASH(MAX_MEM);
		Write_FLASH(RESET_ADR,1);			// write to memory that RESET was done
		Blik_LED();							// blinking LED and stops
											
	}
		
		
   DDRA_BIT1 = 1;						// PTA1 back output

	if (Read_FLASH(RESET_ADR) == 0)		// no reset was done -> send data from mem
	{
		IO_LED = 1;
		Sends_data();						// sends data to the PC
		IO_LED = 0;							// switch off LED
		while(1);							// stop
	}
	
	// go on now, ready 	

//	Delay(10000);

	Write_FLASH(RESET_ADR,0);				// delete reset flag


	I2C_start();							// configure ADC
	IO_LED = I2C_write(0x92);				// ADC's address 
	IO_LED = I2C_write(0x88);				// 15bit resolution
	I2C_stop();
	Delay(10000);

	
	Mem_address = DATA_START;				// sets pointer to data start value
	
	Pressure = Measure_ADC();				// make first measurement and write header
	Write_FLASH(Mem_address,127);
	Write_FLASH(Mem_address+1,((Pressure >> 8) & 0xFF));	// write higher byte
	Write_FLASH(Mem_address+2,(Pressure & 0xFF));	// write lower byte
	Write_FLASH(Mem_address+3,(PERIOD));		// write period value
	
	Previous_pressure = Pressure;
	Mem_address = Mem_address+4;

	Init();

	for (;;)								// stadard operation, wait for Ticks and then measure&write
	{


		if ((Tick >= PERIOD) & (Mem_address < MAX_MEM))

		{
			IO_LED = 1;
			DisableInterrupts();
			Tick = 0;
			Measure_and_write();					// measures pressure and writes to memory
			EnableInterrupts();	
			IO_LED = 0;
		}

	}

}